#pragma once

#include <memory>  // unique_ptr
#include <string>

#include "v3x/BuildCustomAstVisitor.hpp"
#include "v3x/cqasm-parse-result.hpp"

namespace antlr4 {
class ANTLRInputStream;
}
namespace cqasm::v3x::parser {
class CustomErrorListener;
}

namespace cqasm::v3x::parser {

struct ScannerAdaptor {
    virtual ~ScannerAdaptor();

    virtual cqasm::v3x::parser::ParseResult parse() = 0;
};

class ScannerAntlr : public ScannerAdaptor {
    std::unique_ptr<BuildCustomAstVisitor> build_visitor_up_;
    std::unique_ptr<CustomErrorListener> error_listener_up_;

protected:
    cqasm::v3x::parser::ParseResult parse_(antlr4::ANTLRInputStream &is);

public:
    ScannerAntlr(std::unique_ptr<BuildCustomAstVisitor> build_visitor_up,
        std::unique_ptr<CustomErrorListener> error_listener_up);

    ~ScannerAntlr() override;

    cqasm::v3x::parser::ParseResult parse() override = 0;
};

class ScannerAntlrFile : public ScannerAntlr {
    std::string file_path_;

public:
    ScannerAntlrFile(std::unique_ptr<BuildCustomAstVisitor> build_visitor_up,
        std::unique_ptr<CustomErrorListener> error_listener_up, const std::string &file_path);

    ~ScannerAntlrFile() override;

    cqasm::v3x::parser::ParseResult parse() override;
};

class ScannerAntlrString : public ScannerAntlr {
    std::string data_;

public:
    ScannerAntlrString(std::unique_ptr<BuildCustomAstVisitor> build_visitor_up,
        std::unique_ptr<CustomErrorListener> error_listener_up, const std::string &data);

    ~ScannerAntlrString() override;

    cqasm::v3x::parser::ParseResult parse() override;
};

}  // namespace cqasm::v3x::parser
